import psycopg
import json

from .config import get_database_params


def postgres_notify(channel: str, sse_payload: dict):
    connection = psycopg.Connection.connect(
        **get_database_params(),
        autocommit=True,
    )
    with connection.cursor() as cursor:
        cursor.execute(f"NOTIFY {channel}, '{json.dumps(sse_payload)}'")



import json

import uvicorn
import psycopg

from sse_starlette.sse import EventSourceResponse, ServerSentEvent
from starlette.applications import Starlette
from starlette.middleware import Middleware
from starlette.middleware.cors import CORSMiddleware
from starlette.requests import Request
from starlette.routing import Route
from .config import get_database_params, get_allowed_origins, get_debug_value


async def event_publisher(request: Request):
    aconnection = await psycopg.AsyncConnection.connect(
        **get_database_params(),
        autocommit=True,
    )
    channel = request.query_params.get("channel")
    if not channel:
        return

    async with aconnection.cursor() as acursor:
        await acursor.execute(f"LISTEN {channel}")
        generator = aconnection.notifies()
        async for notify_message in generator:
            print(json.loads(notify_message.payload))
            yield ServerSentEvent(**json.loads(notify_message.payload))


async def sse(request: Request):
    return EventSourceResponse(event_publisher(request))


routes = [Route("/", endpoint=sse)]


middleware = [
    Middleware(
        CORSMiddleware,
        allow_origins=get_allowed_origins(),
        allow_credentials=True,
        allow_methods=["GET"],
        allow_headers=["*"],
    )
]

app = Starlette(debug=get_debug_value(), routes=routes, middleware=middleware)


if __name__ == "__main__":
    # pass arguments to uvicorn.run() here
    uvicorn.run("sse_relay_server.main:app", host="0.0.0.0", port=8001)



from dotenv import load_dotenv
import os
import dj_database_url

load_dotenv()


class ConfigurationError(Exception):
    pass


def get_allowed_origins() -> list[str]:
    origins = os.getenv("SSE_ALLOWED_ORIGINS")
    return [] if origins is None else [v.strip() for v in origins.split(",")]


def get_debug_value() -> bool:
    return os.getenv("SSE_RELAY_SERVER_DEBUG") == "True"


def get_database_params() -> dict:
    if not os.getenv("DATABASE_URL"):
        raise ConfigurationError("DATABASE_URL is not set")
    parsed_params = dj_database_url.config()
    if parsed_params["ENGINE"] not in [
        "django.db.backends.postgresql_psycopg2",
        "django.db.backends.postgresql",
    ]:
        raise ConfigurationError("A postgresql database is require")
    return {
        "client_encoding": "UTF8",
        "dbname": parsed_params["NAME"],
        "user": parsed_params["USER"],
        "password": parsed_params["PASSWORD"],
        "host": parsed_params["HOST"],
    }
